local tier_max = settings.startup["machinescaling-tier-assembling-machine-3"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_I1
local pipe_connections_O1
local fluid_box_I
local fluid_box_O

enty_type = "assembling-machine"
machine_name = "assembling-machine-3"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "automation-3" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 150 * math.pow(2,tier)
  pipe_connections_I1 = {}
  pipe_connections_O1 = {}
  for i = 0,tier,1 do
    local pipe_x = -0.5*tier + i
      -- ori: 0
      -- +1: -0.5, 0.5
      -- +2: -1, 0, 1
      -- +3: -1.5, -0.5, 0.5, 1.5
    table.insert( pipe_connections_I1, { type="input",  position = {pipe_x, -0.5-1.5*linescale} } )
    table.insert( pipe_connections_O1, { type="output", position = {pipe_x,  0.5+1.5*linescale} } )
  end
  if pipescaling then
    fluid_box_I =
    {
      base_area = 10,
      height = 1*multscale,
      base_level = -1*multscale,
    }
    fluid_box_O =
    {
      base_area = 10,
      height = 1*multscale,
      base_level = 1*multscale,
    }
  else
    fluid_box_I =
    {
      base_area = 10*multscale,
      height = 1,
      base_level = -1,
    }
    fluid_box_O =
    {
      base_area = 10*multscale,
      height = 1,
      base_level = 1,
    }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "production-machine",
      order = "c[assembling-machine-3]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral","placeable-player", "player-creation"},
      minable = {mining_time = 0.2, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_shift = util.by_pixel(-3*linescale, -12*linescale),
      alert_icon_scale = linescale-1,
      scale_entity_info_icon = true,
      resistances =
      {
        {
          type = "fire",
          percent = 70
        }
      },
      fluid_boxes =
      {
        {
          production_type = "input",
          pipe_picture = assembler3pipepictures(),
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_I.base_area,
          height = fluid_box_I.height,
          base_level = fluid_box_I.base_level,
          pipe_connections = pipe_connections_I1,
          -- pipe_connections = {{ type="input", position = {0, -0.5-1.5*linescale} }},
          secondary_draw_orders = { north = -1 }
        },
        {
          production_type = "output",
          pipe_picture = assembler3pipepictures(),
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O1,
          -- pipe_connections = {{ type="output", position = {0, 0.5+1.5*linescale} }},
          secondary_draw_orders = { north = -1 }
        },
        off_when_no_fluid_recipe = true
      },
      open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
      close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
          {
            filename = "__base__/sound/assembling-machine-t3-1.ogg",
            volume = 0.8
          },
          {
            filename = "__base__/sound/assembling-machine-t3-2.ogg",
            volume = 0.8
          }
        },
        idle_sound = { filename = "__base__/sound/idle1.ogg", volume = 0.6 },
        apparent_volume = 1.5
      },
      collision_box = {{0.3-1.5*linescale, 0.3-1.5*linescale}, {-0.3+1.5*linescale, -0.3+1.5*linescale}},
      -- collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      drawing_box = {{-1.5*linescale, -0.2-1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- drawing_box = {{-1.5, -1.7}, {1.5, 1.5}},
      fast_replaceable_group = "assembling-machine",
      animation =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/assembling-machine-3/assembling-machine-3.png",
            priority = "high",
            width = 108,
            height = 119,
            frame_count = 32,
            line_length = 8,
            shift = util.by_pixel(0, -0.5*linescale),
            scale = linescale*1.1, --1
            animation_speed = 1/multscale,
            hr_version =
            {
              filename = "__base__/graphics/entity/assembling-machine-3/hr-assembling-machine-3.png",
              priority = "high",
              width = 214,
              height = 237,
              frame_count = 32,
              line_length = 8,
              shift = util.by_pixel(0, -0.75*linescale),
              scale = 0.5 * linescale*1.1,
              animation_speed = 1/multscale
            }
          },
          {
            filename = "__base__/graphics/entity/assembling-machine-3/assembling-machine-3-shadow.png",
            priority = "high",
            width = 130,
            height = 82,
            frame_count = 32,
            line_length = 8,
            draw_as_shadow = true,
            shift = util.by_pixel(28*linescale, 4*linescale),
            scale = linescale*1.1, --1
            animation_speed = 1/multscale,
            hr_version =
            {
              filename = "__base__/graphics/entity/assembling-machine-3/hr-assembling-machine-3-shadow.png",
              priority = "high",
              width = 260,
              height = 162,
              frame_count = 32,
              line_length = 8,
              draw_as_shadow = true,
              shift = util.by_pixel(28*linescale, 4*linescale),
              scale = 0.5 * linescale*1.1,
              animation_speed = 1/multscale
            }
          }
        }
      },

      crafting_categories = {"basic-crafting", "crafting", "advanced-crafting", "crafting-with-fluid"},
      crafting_speed = 1.25 * multscale,
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input",
        emissions_per_minute = 2 * multscale
        -- emissions_per_second_per_watt = 2 / 375000
        -- emissions = 0.005
      },
      energy_usage = 375*multscale .. "kW", --"375kW", --"210kW",
      -- ingredient_count = 6,
      module_specification =
      {
        module_slots = 4,
        module_info_icon_shift = {0, 0.8*linescale},
        module_info_icon_scale = 0.5 * linescale
      },
      allowed_effects = {"consumption", "speed", "productivity", "pollution"}
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-a-a"
    },
  }
end
