local tier_max = settings.startup["machinescaling-tier-boiler"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_IO
local pipe_connections_O
local fluid_box
local output_fluid_box

enty_type = "boiler"
machine_name = "boiler"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
-- module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "automation-3" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 50 * math.pow(2,tier)
  pipe_connections_IO = {}
  pipe_connections_O = {}
  for i = 0,tier,1 do
    local pipe_yio = 0.5 + i
      -- ori: 0.5
      -- +1:  0.5, 1.5
      -- +2:  0.5, 1.5, 2.5
      -- +3:  0.5, 1.5, 2.5, 3.5
    local pipe_xo = -0.5*tier + i
      -- ori: 0
      -- +1:  -0.5, 0.5
      -- +2:  -1, 0, 1
      -- +3:  -1.5, -0.5, 0.5, 1.5
    table.insert( pipe_connections_IO, { type = "input-output", position = {-0.5-1.5*linescale, pipe_yio} } )
    table.insert( pipe_connections_IO, { type = "input-output", position = { 0.5+1.5*linescale, pipe_yio} } )
    table.insert( pipe_connections_O,  { type = "output", position = {pipe_xo, -0.5-1*linescale} } )
  end
  if pipescaling then
    fluid_box =
    {
      base_area = 1,
      height = 2*multscale,
      base_level = -1*multscale,
    }
    output_fluid_box =
    {
      base_area = 1,
      height = 2*multscale,
      base_level = 1*multscale,
    }
  else
    fluid_box =
    {
      base_area = 1*multscale,
      height = 2,
      base_level = -1,
    }
    output_fluid_box =
    {
      base_area = 1*multscale,
      height = 2,
      base_level = 1,
    }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "energy",
      order = "b[steam-power]-a[boiler]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "player-creation"},
      minable = {mining_time = 0.2, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      alert_icon_scale = linescale-1,
      -- scale_entity_info_icon = true,
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      mode = "output-to-separate-pipe",
      resistances =
      {
        {
          type = "fire",
          percent = 90
        },
        {
          type = "explosion",
          percent = 30
        },
        {
          type = "impact",
          percent = 30
        }
      },
      collision_box = {{0.21-1.5*linescale, 0.21-1*linescale}, {-0.21+1.5*linescale, -0.21+1*linescale}},
      -- collision_box = {{-1.29, -0.79}, {1.29, 0.79}},
      selection_box = {{-1.5*linescale, -1*linescale}, {1.5*linescale, 1*linescale}},
      -- selection_box = {{-1.5, -1}, {1.5, 1}},
      target_temperature = 165,
      fluid_box =
      {
        base_area = fluid_box.base_area,
        height = fluid_box.height,
        base_level = fluid_box.base_level,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_IO,
        -- pipe_connections =
        -- {
        --   {type = "input-output", position = {-2, 0.5}},
        --   {type = "input-output", position = {2, 0.5}}
        -- },
        production_type = "input-output",
        filter = "water"
      },
      output_fluid_box =
      {
        base_area = output_fluid_box.base_area,
        height = output_fluid_box.height,
        base_level = output_fluid_box.base_level,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_O,
        -- pipe_connections =
        -- {
        --   {type = "output", position = {0, -1.5}}
        -- },
        production_type = "output",
        filter = "steam"
      },
      energy_consumption = 1.8*multscale .. "MW", --"1.8MW",
      energy_source =
      {
        type = "burner",
        fuel_category = "chemical",
        effectivity = 1,
        fuel_inventory_size = 1,
        emissions_per_minute = 30 * multscale,
        -- emissions_per_second_per_watt = 30 / 1800000,
        -- emissions = 0.1 / 6.5,
        smoke =
        {
          {
            name = "smoke",
            north_position = util.by_pixel(-38*linescale, -47.5*linescale),
            south_position = util.by_pixel(38.5*linescale, -32*linescale),
            east_position = util.by_pixel(20*linescale, -70*linescale),
            west_position = util.by_pixel(-19*linescale, -8.5*linescale),
            frequency = 15,
            starting_vertical_speed = 0.0,
            starting_frame_deviation = 60
          }
        }
      },
      working_sound =
      {
        sound =
        {
          filename = "__base__/sound/boiler.ogg",
          volume = 0.8
        },
        max_sounds_per_type = 3
      },

      structure =
      {
        north =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/boiler/boiler-N-idle.png",
              priority = "extra-high",
              width = 131,
              height = 108,
              shift = util.by_pixel(-0.5*linescale, 4*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-N-idle.png",
                priority = "extra-high",
                width = 269,
                height = 221,
                shift = util.by_pixel(-1.25*linescale, 5.25*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-N-shadow.png",
              priority = "extra-high",
              width = 137,
              height = 82,
              scale = linescale, --1
              shift = util.by_pixel(20.5*linescale, 9*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-N-shadow.png",
                priority = "extra-high",
                width = 274,
                height = 164,
                scale = 0.5 * linescale,
                shift = util.by_pixel(20.5*linescale, 9*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        east =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/boiler/boiler-E-idle.png",
              priority = "extra-high",
              width = 105,
              height = 147,
              shift = util.by_pixel(-3.5*linescale, -0.5*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-E-idle.png",
                priority = "extra-high",
                width = 216,
                height = 301,
                shift = util.by_pixel(-3*linescale, 1.25*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-E-shadow.png",
              priority = "extra-high",
              width = 92,
              height = 97,
              scale = linescale, --1
              shift = util.by_pixel(30*linescale, 9.5*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-E-shadow.png",
                priority = "extra-high",
                width = 184,
                height = 194,
                scale = 0.5 * linescale,
                shift = util.by_pixel(30*linescale, 9.5*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        south =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/boiler/boiler-S-idle.png",
              priority = "extra-high",
              width = 128,
              height = 95,
              shift = util.by_pixel(3*linescale, 12.5*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-S-idle.png",
                priority = "extra-high",
                width = 260,
                height = 192,
                shift = util.by_pixel(4*linescale, 13*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-S-shadow.png",
              priority = "extra-high",
              width = 156,
              height = 66,
              scale = linescale, --1
              shift = util.by_pixel(30*linescale, 16*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-S-shadow.png",
                priority = "extra-high",
                width = 311,
                height = 131,
                scale = 0.5 * linescale,
                shift = util.by_pixel(29.75*linescale, 15.75*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        west =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/boiler/boiler-W-idle.png",
              priority = "extra-high",
              width = 96,
              height = 132,
              shift = util.by_pixel(1*linescale, 5*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-W-idle.png",
                priority = "extra-high",
                width = 196,
                height = 273,
                shift = util.by_pixel(1.5*linescale, 7.75*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-W-shadow.png",
              priority = "extra-high",
              width = 103,
              height = 109,
              scale = linescale, --1
              shift = util.by_pixel(19.5*linescale, 6.5*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-W-shadow.png",
                priority = "extra-high",
                width = 206,
                height = 218,
                scale = 0.5 * linescale,
                shift = util.by_pixel(19.5*linescale, 6.5*linescale),
                draw_as_shadow = true
              }
            }
          }
        }
      },

      patch =
      {
        east =
        {
          filename = "__base__/graphics/entity/boiler/boiler-E-patch.png",
          priority = "extra-high",
          width = 3,
          height = 17,
          shift = util.by_pixel(33.5*linescale, -13.5*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-E-patch.png",
            width = 6,
            height = 36,
            shift = util.by_pixel(33.5*linescale, -13.5*linescale),
            scale = 0.5 * linescale
          }
        }
      },

      fire_flicker_enabled = true,
      fire =
      {
        north =
        {
          filename = "__base__/graphics/entity/boiler/boiler-N-fire.png",
          priority = "extra-high",
          frame_count = 64,
          line_length = 8,
          width = 12,
          height = 13,
          animation_speed = 0.5,
          shift = util.by_pixel(0*linescale, -8.5*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-N-fire.png",
            priority = "extra-high",
            frame_count = 64,
            line_length = 8,
            width = 26,
            height = 26,
            animation_speed = 0.5,
            shift = util.by_pixel(0*linescale, -8.5*linescale),
            scale = 0.5 * linescale
          }
        },
        east =
        {
          filename = "__base__/graphics/entity/boiler/boiler-E-fire.png",
          priority = "extra-high",
          frame_count = 64,
          line_length = 8,
          width = 14,
          height = 14,
          animation_speed = 0.5,
          shift = util.by_pixel(-10*linescale, -22*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-E-fire.png",
            priority = "extra-high",
            frame_count = 64,
            line_length = 8,
            width = 28,
            height = 28,
            animation_speed = 0.5,
            shift = util.by_pixel(-9.5*linescale, -22*linescale),
            scale = 0.5 * linescale
          }
        },
        south =
        {
          filename = "__base__/graphics/entity/boiler/boiler-S-fire.png",
          priority = "extra-high",
          frame_count = 64,
          line_length = 8,
          width = 12,
          height = 9,
          animation_speed = 0.5,
          shift = util.by_pixel(-1*linescale, -26.5*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-S-fire.png",
            priority = "extra-high",
            frame_count = 64,
            line_length = 8,
            width = 26,
            height = 16,
            animation_speed = 0.5,
            shift = util.by_pixel(-1*linescale, -26.5*linescale),
            scale = 0.5 * linescale
          }
        },
        west =
        {
          filename = "__base__/graphics/entity/boiler/boiler-W-fire.png",
          priority = "extra-high",
          frame_count = 64,
          line_length = 8,
          width = 14,
          height = 14,
          animation_speed = 0.5,
          shift = util.by_pixel(13*linescale, -23*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-W-fire.png",
            priority = "extra-high",
            frame_count = 64,
            line_length = 8,
            width = 30,
            height = 29,
            animation_speed = 0.5,
            shift = util.by_pixel(13*linescale, -23.25*linescale),
            scale = 0.5 * linescale
          }
        }
      },

      fire_glow_flicker_enabled = true,

      fire_glow =
      {
        north =
        {
          filename = "__base__/graphics/entity/boiler/boiler-N-light.png",
          priority = "extra-high",
          frame_count = 1,
          width = 100,
          height = 87,
          shift = util.by_pixel(-1*linescale, -6.5*linescale),
          blend_mode = "additive",
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-N-light.png",
            priority = "extra-high",
            frame_count = 1,
            width = 200,
            height = 173,
            shift = util.by_pixel(-1*linescale, -6.75*linescale),
            blend_mode = "additive",
            scale = 0.5 * linescale
          }
        },
        east =
        {
          filename = "__base__/graphics/entity/boiler/boiler-E-light.png",
          priority = "extra-high",
          frame_count = 1,
          width = 70,
          height = 122,
          shift = util.by_pixel(0, -13*linescale),
          blend_mode = "additive",
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-E-light.png",
            priority = "extra-high",
            frame_count = 1,
            width = 139,
            height = 244,
            shift = util.by_pixel(0.25*linescale, -13*linescale),
            blend_mode = "additive",
            scale = 0.5 * linescale
          }
        },
        south =
        {
          filename = "__base__/graphics/entity/boiler/boiler-S-light.png",
          priority = "extra-high",
          frame_count = 1,
          width = 100,
          height = 81,
          shift = util.by_pixel(1*linescale, 5.5*linescale),
          blend_mode = "additive",
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-S-light.png",
            priority = "extra-high",
            frame_count = 1,
            width = 200,
            height = 162,
            shift = util.by_pixel(1*linescale, 5.5*linescale),
            blend_mode = "additive",
            scale = 0.5 * linescale
          }
        },
        west =
        {
          filename = "__base__/graphics/entity/boiler/boiler-W-light.png",
          priority = "extra-high",
          frame_count = 1,
          width = 68,
          height = 109,
          shift = util.by_pixel(2*linescale, -6.5*linescale),
          blend_mode = "additive",
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/boiler/hr-boiler-W-light.png",
            priority = "extra-high",
            frame_count = 1,
            width = 136,
            height = 217,
            shift = util.by_pixel(2*linescale, -6.25*linescale),
            blend_mode = "additive",
            scale = 0.5 * linescale
          }
        }
      },
      burning_cooldown = 20
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        -- {"speed-module-3", module_ing},
        -- {"productivity-module-3", module_ing},
        -- {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-o-a"
    },
  }
end
