local tier_max = settings.startup["machinescaling-tier-centrifuge"].value

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

enty_type = "assembling-machine"
machine_name = "centrifuge"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "nuclear-power" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 150 * math.pow(2,tier)

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "production-machine",
      order = "g[centrifuge]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "placeable-player", "player-creation"},
      minable = {mining_time = 0.1, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_scale = linescale-1,
      scale_entity_info_icon = true,
      resistances =
      {
        {
          type = "fire",
          percent = 70
        }
      },
      collision_box = {{0.3-1.5*linescale, 0.3-1.5*linescale}, {-0.3+1.5*linescale, -0.3+1.5*linescale}},
      -- collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      drawing_box = {{-1.5*linescale, -0.7-1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- drawing_box = {{-1.5, -2.2}, {1.5, 1.5}},

      always_draw_idle_animation = true,
      idle_animation =
      {
        layers =
        {
          -- Centrifuge A
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-C.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 119,
            height = 107,
            frame_count = 64,
            shift = util.by_pixel(-0.5*linescale, -26.5*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-C.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 237,
              height = 214,
              frame_count = 64,
              shift = util.by_pixel(-0.25*linescale, -26.5*linescale)
            }
          },
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-C-shadow.png",
            draw_as_shadow = true,
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 132,
            height = 74,
            frame_count = 64,
            shift = util.by_pixel(20*linescale, -10*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-C-shadow.png",
              draw_as_shadow = true,
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 279,
              height = 152,
              frame_count = 64,
              shift = util.by_pixel(16.75*linescale, -10*linescale)
            }
          },
          -- Centrifuge B
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-B.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 78,
            height = 117,
            frame_count = 64,
            shift = util.by_pixel(23*linescale, 6.5*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-B.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 156,
              height = 234,
              frame_count = 64,
              shift = util.by_pixel(23*linescale, 6.5*linescale)
            }
          },
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-B-shadow.png",
            draw_as_shadow = true,
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 124,
            height = 74,
            frame_count = 64,
            shift = util.by_pixel(63*linescale, 16*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-B-shadow.png",
              draw_as_shadow = true,
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 251,
              height = 149,
              frame_count = 64,
              shift = util.by_pixel(63.25*linescale, 15.25*linescale)
            }
          },
          -- Centrifuge A
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-A.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 70,
            height = 123,
            frame_count = 64,
            shift = util.by_pixel(-26*linescale, 3.5*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-A.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 139,
              height = 246,
              frame_count = 64,
              shift = util.by_pixel(-26.25*linescale, 3.5*linescale)
            }
          },
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-A-shadow.png",
            priority = "high",
            draw_as_shadow = true,
            scale = linescale, --1
            animation_speed = 1/multscale,
            line_length = 8,
            width = 108,
            height = 54,
            frame_count = 64,
            shift = util.by_pixel(6*linescale, 27*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-A-shadow.png",
              priority = "high",
              draw_as_shadow = true,
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              line_length = 8,
              width = 230,
              height = 124,
              frame_count = 64,
              shift = util.by_pixel(8.5*linescale, 23.5*linescale)
            }
          }
        }
      },

      animation =
      {
        layers =
        {
          -- Centrifuge C
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-C-light.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            blend_mode = "additive", -- centrifuge
            line_length = 8,
            width = 96,
            height = 104,
            frame_count = 64,
            shift = util.by_pixel(0, -27*linescale),

            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-C-light.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              blend_mode = "additive", -- centrifuge
              line_length = 8,
              width = 190,
              height = 207,
              frame_count = 64,
              shift = util.by_pixel(0, -27.25*linescale)
            }
          },
          -- Centrifuge B
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-B-light.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            blend_mode = "additive", -- centrifuge
            line_length = 8,
            width = 65,
            height = 103,
            frame_count = 64,
            shift = util.by_pixel(16.5*linescale, 0.5*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-B-light.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              blend_mode = "additive", -- centrifuge
              line_length = 8,
              width = 131,
              height = 206,
              frame_count = 64,
              shift = util.by_pixel(16.75*linescale, 0.5*linescale)
            }
          },
          -- Centrifuge A
          {
            filename = "__base__/graphics/entity/centrifuge/centrifuge-A-light.png",
            priority = "high",
            scale = linescale, --1
            animation_speed = 1/multscale,
            blend_mode = "additive", -- centrifuge
            line_length = 8,
            width = 55,
            height = 98,
            frame_count = 64,
            shift = util.by_pixel(-23.5*linescale, -2*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-A-light.png",
              priority = "high",
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              blend_mode = "additive", -- centrifuge
              line_length = 8,
              width = 108,
              height = 197,
              frame_count = 64,
              shift = util.by_pixel(-23.5*linescale, -1.75*linescale)
            }
          }
        }
      },

      working_visualisations =
      {
        {
          effect = "uranium-glow", -- changes alpha based on energy source light intensity
          light = {intensity = 0.6, size = 9.9, shift = {0.0, 0.0}, color = {r = 0.0, g = 1.0, b = 0.0}}
        }
      },

      -- working_visualisations_disabled =
      -- {
      --   {
      --     north_position = {0.0, 0.0},
      --     east_position = {0.0, 0.0},
      --     south_position = {0.0, 0.0},
      --     west_position = {0.0, 0.0},
      --     animation =
      --     {
      --       filename = "__base__/graphics/entity/centrifuge/centrifuge-C-light.png",
      --       priority = "high",
      --       scale = linescale, --1
      --       blend_mode = "additive", -- centrifuge
      --       line_length = 8,
      --       width = 96,
      --       height = 104,
      --       frame_count = 64,
      --       shift = util.by_pixel(0, -27*linescale),

      --       hr_version =
      --       {
      --         filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-C-light.png",
      --         priority = "high",
      --         scale = 0.5 * linescale,
      --         blend_mode = "additive", -- centrifuge
      --         line_length = 8,
      --         width = 190,
      --         height = 207,
      --         frame_count = 64,
      --         shift = util.by_pixel(0, -27.25*linescale)
      --       }
      --     },
      --     --light = {intensity = 1, size = 1, color = {r = 1.0, g = 1.0, b = 1.0}}
      --   },
      --   {
      --     north_position = {0.0, 0.0},
      --     east_position = {0.0, 0.0},
      --     south_position = {0.0, 0.0},
      --     west_position = {0.0, 0.0},
      --     animation =
      --     {
      --       filename = "__base__/graphics/entity/centrifuge/centrifuge-B-light.png",
      --       priority = "high",
      --       scale = linescale, --1
      --       blend_mode = "additive", -- centrifuge
      --       line_length = 8,
      --       width = 65,
      --       height = 103,
      --       frame_count = 64,
      --       shift = util.by_pixel(16.5*linescale, 0.5*linescale),
      --       hr_version =
      --       {
      --         filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-B-light.png",
      --         priority = "high",
      --         scale = 0.5 * linescale,
      --         blend_mode = "additive", -- centrifuge
      --         line_length = 8,
      --         width = 131,
      --         height = 206,
      --         frame_count = 64,
      --         shift = util.by_pixel(16.75*linescale, 0.5*linescale)
      --       }
      --     }
      --   },
      --   {
      --     north_position = {0.0, 0.0},
      --     east_position = {0.0, 0.0},
      --     south_position = {0.0, 0.0},
      --     west_position = {0.0, 0.0},
      --     animation =
      --     {
      --       filename = "__base__/graphics/entity/centrifuge/centrifuge-A-light.png",
      --       priority = "high",
      --       scale = linescale, --1
      --       blend_mode = "additive", -- centrifuge
      --       line_length = 8,
      --       width = 55,
      --       height = 98,
      --       frame_count = 64,
      --       shift = util.by_pixel(-23.5*linescale, -2*linescale),
      --       hr_version =
      --       {
      --         filename = "__base__/graphics/entity/centrifuge/hr-centrifuge-A-light.png",
      --         priority = "high",
      --         scale = 0.5 * linescale,
      --         blend_mode = "additive", -- centrifuge
      --         line_length = 8,
      --         width = 108,
      --         height = 197,
      --         frame_count = 64,
      --         shift = util.by_pixel(-23.5*linescale, -1.75*linescale)
      --       }
      --     }
      --   }
      -- },

      open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
      close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
          {
            filename = "__base__/sound/assembling-machine-t2-1.ogg",
            volume = 0.8
          },
          {
            filename = "__base__/sound/assembling-machine-t2-2.ogg",
            volume = 0.8
          }
        },
        idle_sound = { filename = "__base__/sound/idle1.ogg", volume = 0.6 },
        apparent_volume = 1.5
      },
      crafting_speed = multscale,
      crafting_categories = {"centrifuging"},
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input",
        emissions_per_minute = 4 * multscale
        -- emissions_per_second_per_watt = 4 / 350000
        -- emissions = 0.04 / 2.5
      },
      energy_usage = 350*multscale .. "kW", --"350kW",
      -- ingredient_count = 2,
      module_specification =
      {
        module_slots = 2,
        module_info_icon_shift = {0, 0.8*linescale},
        module_info_icon_scale = 0.5 * linescale
      },
      allowed_effects = {"consumption", "speed", "productivity", "pollution"}
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-a-g"
    },
  }
end
