local tier_max = settings.startup["machinescaling-tier-chemical-plant"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_I1
local pipe_connections_I2
local pipe_connections_O1
local pipe_connections_O2
local fluid_box_I
local fluid_box_O

enty_type = "assembling-machine"
machine_name = "chemical-plant"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "oil-processing" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 150 * math.pow(2,tier)
  pipe_connections_I1 = {}
  pipe_connections_I2 = {}
  pipe_connections_O1 = {}
  pipe_connections_O2 = {}
  for i = 0,tier,1 do
    local pipe_x = 1+0.5*tier + i
      -- ori: 1
      -- +1:  1.5, 2.5
      -- +2:  2, 3, 4
      -- +3:  2.5, 3.5, 4.5, 5.5
    table.insert( pipe_connections_I1, { type="input", position = {-pipe_x, -0.5-1.5*linescale} } )
    table.insert( pipe_connections_I2, { type="input", position = { pipe_x, -0.5-1.5*linescale} } )
    table.insert( pipe_connections_O1, { position = {-pipe_x,  0.5+1.5*linescale} } )
    table.insert( pipe_connections_O2, { position = { pipe_x,  0.5+1.5*linescale} } )
  end
  if pipescaling then
    fluid_box_I =
    {
      base_area = 10,
      height = 1*multscale,
      base_level = -1*multscale,
    }
    fluid_box_O =
    {
      base_area = 1,
      height = 1*multscale,
      base_level = 1*multscale,
    }
  else
    fluid_box_I =
    {
      base_area = 10*multscale,
      height = 1,
      base_level = -1,
    }
    fluid_box_O =
    {
      base_area = 1*multscale,
      height = 1,
      base_level = 1,
    }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "production-machine",
      order = "e[chemical-plant]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral","placeable-player", "player-creation"},
      minable = {mining_time = 0.1, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_shift = util.by_pixel(-3*linescale, -12*linescale),
      alert_icon_scale = linescale-1,
      scale_entity_info_icon = true,
      collision_box = {{0.3-1.5*linescale, 0.3-1.5*linescale}, {-0.3+1.5*linescale, -0.3+1.5*linescale}},
      -- collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      drawing_box = {{-1.5*linescale, -0.4-1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- drawing_box = {{-1.5, -1.9}, {1.5, 1.5}},
      module_specification =
      {
        module_slots = 3,
        module_info_icon_shift = {0, 0.8*linescale},
        module_info_icon_scale = 0.5 * linescale
      },
      allowed_effects = {"consumption", "speed", "productivity", "pollution"},

      animation = make_4way_animation_from_spritesheet({ layers =
      {
        {
          filename = "__base__/graphics/entity/chemical-plant/chemical-plant.png",
          width = 108,
          height = 148,
          frame_count = 24,
          line_length = 12,
          shift = util.by_pixel(1*linescale, -9*linescale),
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant.png",
            width = 220,
            height = 292,
            frame_count = 24,
            line_length = 12,
            shift = util.by_pixel(0.5*linescale, -9*linescale),
            scale = 0.5 * linescale
            }
        },
        {
          filename = "__base__/graphics/entity/chemical-plant/chemical-plant-shadow.png",
          width = 154,
          height = 112,
          repeat_count = 24,
          frame_count = 1,
          shift = util.by_pixel(28*linescale, 6*linescale),
          scale = linescale, --1
          draw_as_shadow = true,
          hr_version =
          {
            filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-shadow.png",
            width = 312,
            height = 222,
            repeat_count = 24,
            frame_count = 1,
            shift = util.by_pixel(27*linescale, 6*linescale),
            draw_as_shadow = true,
            scale = 0.5 * linescale
            }
        }
      }}),
      working_visualisations =
      {
            -- animation_speed = 1/multscale,
              -- animation_speed = 1/multscale
        {
          apply_recipe_tint = "primary",
          north_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-liquid-north.png",
            frame_count = 24,
            line_length = 6,
            width = 32,
            height = 24,
            animation_speed = 1/multscale,
            shift = util.by_pixel(24*linescale, 14*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-liquid-north.png",
              frame_count = 24,
              line_length = 6,
              width = 66,
              height = 44,
              animation_speed = 1/multscale,
              shift = util.by_pixel(23*linescale, 15*linescale),
              scale = 0.5 * linescale
            }
          },
          east_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-liquid-east.png",
            frame_count = 24,
            line_length = 6,
            width = 36,
            height = 18,
            animation_speed = 1/multscale,
            shift = util.by_pixel(0, 22*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-liquid-east.png",
              frame_count = 24,
              line_length = 6,
              width = 70,
              height = 36,
              animation_speed = 1/multscale,
              shift = util.by_pixel(0, 22*linescale),
              scale = 0.5 * linescale
            }
          },
          south_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-liquid-south.png",
            frame_count = 24,
            line_length = 6,
            width = 34,
            height = 24,
            animation_speed = 1/multscale,
            shift = util.by_pixel(0, 16*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-liquid-south.png",
              frame_count = 24,
              line_length = 6,
              width = 66,
              height = 42,
              animation_speed = 1/multscale,
              shift = util.by_pixel(0, 17*linescale),
              scale = 0.5 * linescale
            }
          },
          west_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-liquid-west.png",
            frame_count = 24,
            line_length = 6,
            width = 38,
            height = 20,
            animation_speed = 1/multscale,
            shift = util.by_pixel(-10*linescale, 12*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-liquid-west.png",
              frame_count = 24,
              line_length = 6,
              width = 74,
              height = 36,
              animation_speed = 1/multscale,
              shift = util.by_pixel(-10*linescale, 13*linescale),
              scale = 0.5 * linescale
            }
          }
        },
        {
          apply_recipe_tint = "secondary",
          north_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-foam-north.png",
            frame_count = 24,
            line_length = 6,
            width = 32,
            height = 22,
            animation_speed = 1/multscale,
            shift = util.by_pixel(24*linescale, 14*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-foam-north.png",
              frame_count = 24,
              line_length = 6,
              width = 62,
              height = 42,
              animation_speed = 1/multscale,
              shift = util.by_pixel(24*linescale, 15*linescale),
              scale = 0.5 * linescale
            }
          },
          east_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-foam-east.png",
            frame_count = 24,
            line_length = 6,
            width = 34,
            height = 18,
            animation_speed = 1/multscale,
            shift = util.by_pixel(0, 22*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-foam-east.png",
              frame_count = 24,
              line_length = 6,
              width = 68,
              height = 36,
              animation_speed = 1/multscale,
              shift = util.by_pixel(0, 22*linescale),
              scale = 0.5 * linescale
            }
          },
          south_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-foam-south.png",
            frame_count = 24,
            line_length = 6,
            width = 32,
            height = 18,
            animation_speed = 1/multscale,
            shift = util.by_pixel(0, 18*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-foam-south.png",
              frame_count = 24,
              line_length = 6,
              width = 60,
              height = 40,
              animation_speed = 1/multscale,
              shift = util.by_pixel(1*linescale, 17*linescale),
              scale = 0.5 * linescale
            }
          },
          west_animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-foam-west.png",
            frame_count = 24,
            line_length = 6,
            width = 36,
            height = 16,
            animation_speed = 1/multscale,
            shift = util.by_pixel(-10*linescale, 14*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-foam-west.png",
              frame_count = 24,
              line_length = 6,
              width = 68,
              height = 28,
              animation_speed = 1/multscale,
              shift = util.by_pixel(-9*linescale, 15*linescale),
              scale = 0.5 * linescale
            }
          }
        },
        {
          apply_recipe_tint = "tertiary",
          fadeout = true,
          constant_speed = true,
          north_position = util.by_pixel_hr(-30*linescale, -161*linescale),
          east_position = util.by_pixel_hr(29*linescale, -150*linescale),
          south_position = util.by_pixel_hr(12*linescale, -134*linescale),
          west_position = util.by_pixel_hr(-32*linescale, -130*linescale),
          animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-smoke-outer.png",
            frame_count = 47,
            line_length = 16,
            width = 46,
            height = 94,
            animation_speed = 0.5,
            shift = util.by_pixel(-2*linescale, -40*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-smoke-outer.png",
              frame_count = 47,
              line_length = 16,
              width = 90,
              height = 188,
              animation_speed = 0.5,
              shift = util.by_pixel(-2*linescale, -40*linescale),
              scale = 0.5 * linescale
            }
          }
        },
        {
          apply_recipe_tint = "quaternary",
          fadeout = true,
          constant_speed = true,
          north_position = util.by_pixel_hr(-30*linescale, -161*linescale),
          east_position = util.by_pixel_hr(29*linescale, -150*linescale),
          south_position = util.by_pixel_hr(12*linescale, -134*linescale),
          west_position = util.by_pixel_hr(-32*linescale, -130*linescale),
          animation =
          {
            filename = "__base__/graphics/entity/chemical-plant/chemical-plant-smoke-inner.png",
            frame_count = 47,
            line_length = 16,
            width = 20,
            height = 42,
            animation_speed = 0.5,
            shift = util.by_pixel(0, -14*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/chemical-plant/hr-chemical-plant-smoke-inner.png",
              frame_count = 47,
              line_length = 16,
              width = 40,
              height = 84,
              animation_speed = 0.5,
              shift = util.by_pixel(0, -14*linescale),
              scale = 0.5 * linescale
            }
          }
        }
      },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
          {
            filename = "__base__/sound/chemical-plant.ogg",
            volume = 0.8
          }
        },
        idle_sound = { filename = "__base__/sound/idle1.ogg", volume = 0.6 },
        apparent_volume = 1.5
      },
      crafting_speed = multscale,
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input",
        emissions_per_minute = 4 * multscale
      },
      energy_usage = 210*multscale .. "kW", --"210kW",
      crafting_categories = {"chemistry"},
      fluid_boxes =
      {
        {
          production_type = "input",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_I.base_area,
          height = fluid_box_I.height,
          base_level = fluid_box_I.base_level,
          pipe_connections = pipe_connections_I1,
          -- pipe_connections = {{ type="input", position = {-1, -0.5-1.5*linescale} }}
        },
        {
          production_type = "input",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_I.base_area,
          height = fluid_box_I.height,
          base_level = fluid_box_I.base_level,
          pipe_connections = pipe_connections_I2,
          -- pipe_connections = {{ type="input", position = {1, -0.5-1.5*linescale} }}
        },
        {
          production_type = "output",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O1,
          -- pipe_connections = {{ position = {-1,  0.5+1.5*linescale} }}
        },
        {
          production_type = "output",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O2,
          -- pipe_connections = {{ position = {1,  0.5+1.5*linescale} }}
        }
      }
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-a-d"
    },
  }
end
