local tier_max = settings.startup["machinescaling-tier-electric-furnace"].value

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

enty_type = "furnace"
machine_name = "electric-furnace"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "advanced-material-processing-2" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 250 * math.pow(2,tier)

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "smelting-machine",
      order = "c[electric-furnace]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "placeable-player", "player-creation"},
      minable = {mining_time = 0.2, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_scale = linescale-1,
      scale_entity_info_icon = true,
      collision_box = {{0.3-1.5*linescale, 0.3-1.5*linescale}, {-0.3+1.5*linescale, -0.3+1.5*linescale}},
      -- collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      resistances =
      {
        {
          type = "fire",
          percent = 80
        }
      },
      module_specification =
      {
        module_slots = 2,
        module_info_icon_shift = {0, 0.8*linescale},
        module_info_icon_scale = 0.5 * linescale
      },
      allowed_effects = {"consumption", "speed", "productivity", "pollution"},
      crafting_categories = {"smelting"},
      result_inventory_size = 1,
      crafting_speed = 2 * multscale,
      energy_usage = 180*multscale .. "kW", --"180kW",
      source_inventory_size = 1,
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input",
        emissions_per_minute = 1 * multscale
        -- emissions_per_second_per_watt = 1 / 180000
        -- emissions = 0.005
      },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
          filename = "__base__/sound/electric-furnace.ogg",
          volume = 0.7
        },
        apparent_volume = 1.5
      },
      animation =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/electric-furnace/electric-furnace-base.png",
            priority = "high",
            width = 129,
            height = 100,
            frame_count = 1,
            shift = {0.421875*linescale, 0},
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/electric-furnace/hr-electric-furnace.png",
              priority = "high",
              width = 239,
              height = 219,
              frame_count = 1,
              shift = util.by_pixel(0.75*linescale, 5.75*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/electric-furnace/electric-furnace-shadow.png",
            priority = "high",
            width = 129,
            height = 100,
            frame_count = 1,
            shift = {0.421875*linescale, 0},
            draw_as_shadow = true,
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/electric-furnace/hr-electric-furnace-shadow.png",
              priority = "high",
              width = 227,
              height = 171,
              frame_count = 1,
              draw_as_shadow = true,
              shift = util.by_pixel(11.25*linescale, 7.75*linescale),
              scale = 0.5 * linescale
            }
          }
        }
      },
      working_visualisations =
      {
        {
          animation =
          {
            filename = "__base__/graphics/entity/electric-furnace/electric-furnace-heater.png",
            priority = "high",
            width = 25,
            height = 15,
            frame_count = 12,
            animation_speed = 0.5,
            shift = {0.015625*linescale, 0.890625*linescale},
            scale = linescale, --1
            animation_speed = 1/multscale,
            hr_version =
            {
              filename = "__base__/graphics/entity/electric-furnace/hr-electric-furnace-heater.png",
              priority = "high",
              width = 60,
              height = 56,
              frame_count = 12,
              animation_speed = 0.5,
              shift = util.by_pixel(1.75*linescale, 32.75*linescale),
              scale = 0.5 * linescale,
              animation_speed = 1/multscale
            }
          },
          light = {intensity = 0.4, size = 6, shift = {0.0, 1.0}, color = {r = 1.0, g = 1.0, b = 1.0}}
        },
        {
          animation =
          {
            filename = "__base__/graphics/entity/electric-furnace/electric-furnace-propeller-1.png",
            priority = "high",
            width = 19,
            height = 13,
            frame_count = 4,
            animation_speed = 0.5,
            shift = {-0.671875*linescale, -0.640625*linescale},
            scale = linescale, --1
            animation_speed = 1/multscale,
            hr_version =
            {
              filename = "__base__/graphics/entity/electric-furnace/hr-electric-furnace-propeller-1.png",
              priority = "high",
              width = 37,
              height = 25,
              frame_count = 4,
              animation_speed = 0.5,
              shift = util.by_pixel(-20.5*linescale, -18.5*linescale),
              scale = 0.5 * linescale,
              animation_speed = 1/multscale
            }
          }
        },
        {
          animation =
          {
            filename = "__base__/graphics/entity/electric-furnace/electric-furnace-propeller-2.png",
            priority = "high",
            width = 12,
            height = 9,
            frame_count = 4,
            animation_speed = 0.5,
            shift = {0.0625*linescale, -1.234375*linescale},
            scale = linescale, --1
            animation_speed = 1/multscale,
            hr_version =
            {
              filename = "__base__/graphics/entity/electric-furnace/hr-electric-furnace-propeller-2.png",
              priority = "high",
              width = 23,
              height = 15,
              frame_count = 4,
              animation_speed = 0.5,
              shift = util.by_pixel(3.5*linescale, -38*linescale),
              scale = 0.5 * linescale,
              animation_speed = 1/multscale
            }
          }
        }
      },
      fast_replaceable_group = "furnace"
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-c"
    },
  }
end
