local tier_max = settings.startup["machinescaling-tier-heat-exchanger"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_IO
local pipe_connections_O
local connections_H
local fluid_box
local output_fluid_box

enty_type = "boiler"
machine_name = "heat-exchanger"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
-- module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "nuclear-power" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 200 * math.pow(2,tier)
  pipe_connections_IO = {}
  pipe_connections_O = {}
  connections_H = {}
  for i = 0,tier,1 do
    local pipe_yio = 0.5 + i
      -- ori: 0.5
      -- +1:  0.5, 1.5
      -- +2:  0.5, 1.5, 2.5
      -- +3:  0.5, 1.5, 2.5, 3.5
    local pipe_xo = -0.5*tier + i
      -- ori: 0
      -- +1:  -0.5, 0.5
      -- +2:  -1, 0, 1
      -- +3:  -1.5, -0.5, 0.5, 1.5
    table.insert( pipe_connections_IO, { type = "input-output", position = {-0.5-1.5*linescale, pipe_yio} } )
    table.insert( pipe_connections_IO, { type = "input-output", position = { 0.5+1.5*linescale, pipe_yio} } )
    table.insert( pipe_connections_O,  { type = "output", position = {pipe_xo, -0.5-1*linescale} } )
    table.insert( connections_H,  { position = {pipe_xo, -0.5+1*linescale}, direction = defines.direction.south } )
  end
  if pipescaling then
    fluid_box =
    {
      base_area = 1,
      height = 2*multscale,
      base_level = -1*multscale,
    }
    output_fluid_box =
    {
      base_area = 1,
      height = 2*multscale,
      base_level = 1*multscale,
    }
  else
    fluid_box =
    {
      base_area = 1*multscale,
      height = 2,
      base_level = -1,
    }
    output_fluid_box =
    {
      base_area = 1*multscale,
      height = 2,
      base_level = 1,
    }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "energy",
      order = "f[nuclear-energy]-b[heat-exchanger]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "player-creation"},
      minable = {mining_time = 0.1, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "medium-remnants",
      alert_icon_scale = linescale-1,
      -- scale_entity_info_icon = true,
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      mode = "output-to-separate-pipe",
      resistances =
      {
        {
          type = "fire",
          percent = 90
        },
        {
          type = "explosion",
          percent = 30
        },
        {
          type = "impact",
          percent = 30
        }
      },
      collision_box = {{0.21-1.5*linescale, 0.21-1*linescale}, {-0.21+1.5*linescale, -0.21+1*linescale}},
      -- collision_box = {{-1.29, -0.79}, {1.29, 0.79}},
      selection_box = {{-1.5*linescale, -1*linescale}, {1.5*linescale, 1*linescale}},
      -- selection_box = {{-1.5, -1}, {1.5, 1}},
      target_temperature = 500,
      fluid_box =
      {
        base_area = fluid_box.base_area,
        height = fluid_box.height,
        base_level = fluid_box.base_level,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_IO,
        -- pipe_connections =
        -- {
        --   {type = "input-output", position = {-2, 0.5}},
        --   {type = "input-output", position = {2, 0.5}}
        -- },
        production_type = "input-output",
        filter = "water"
      },
      output_fluid_box =
      {
        base_area = output_fluid_box.base_area,
        height = output_fluid_box.height,
        base_level = output_fluid_box.base_level,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_O,
        -- pipe_connections =
        -- {
        --   {type = "output", position = {0, -1.5}}
        -- },
        production_type = "output",
        filter = "steam"
      },
      energy_consumption = 10*multscale .. "MW", --"10MW",
      energy_source =
      {
        type = "heat",
        max_temperature = 1000,
        specific_heat = 1*multscale .. "MJ", --"1MJ",
        max_transfer = 2*multscale .. "GW", --"2GW",
        min_working_temperature = 500,
        minimum_glow_temperature = 350,
        connections = connections_H,
        -- connections =
        -- {
        --   {
        --     position = {0, 0.5},
        --     direction = defines.direction.south
        --   }
        -- },
        pipe_covers =
          make_4way_animation_from_spritesheet(
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-endings.png",
            line_length = 4,
            width = 32,
            height = 32,
            direction_count = 4,
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-endings.png",
              line_length = 4,
              width = 64,
              height = 64,
              direction_count = 4,
              scale = 0.5 * linescale
            }
          }),
        heat_pipe_covers =
          make_4way_animation_from_spritesheet(
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-endings-heated.png",
            width = 32,
            height = 32,
            direction_count = 4,
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-endings-heated.png",
              width = 64,
              height = 64,
              direction_count = 4,
              scale = 0.5 * linescale
            }
          }),
        heat_picture =
        {
          north =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-N-heated.png",
            priority = "extra-high",
            width = 24,
            height = 48,
            shift = util.by_pixel(-1*linescale, 8*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-N-heated.png",
              priority = "extra-high",
              width = 44,
              height = 96,
              shift = util.by_pixel(-0.5*linescale, 8.5*linescale),
              scale = 0.5 * linescale
            }
          },
          east =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-E-heated.png",
            priority = "extra-high",
            width = 40,
            height = 40,
            shift = util.by_pixel(-21*linescale, -13*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-E-heated.png",
              priority = "extra-high",
              width = 80,
              height = 80,
              shift = util.by_pixel(-21*linescale, -13*linescale),
              scale = 0.5 * linescale
            }
          },
          south =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-S-heated.png",
            priority = "extra-high",
            width = 16,
            height = 20,
            shift = util.by_pixel(-1*linescale, -30*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-S-heated.png",
              priority = "extra-high",
              width = 28,
              height = 40,
              shift = util.by_pixel(-1*linescale, -30*linescale),
              scale = 0.5 * linescale
            }
          },
          west =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-W-heated.png",
            priority = "extra-high",
            width = 32,
            height = 40,
            shift = util.by_pixel(23*linescale, -13*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-W-heated.png",
              priority = "extra-high",
              width = 64,
              height = 76,
              shift = util.by_pixel(23*linescale, -13*linescale),
              scale = 0.5 * linescale
            }
          },
        },
        heat_glow =
        {
          north =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-N-glow.png",
            priority = "extra-high",
            width = 38,
            height = 70,
            tint = heat_glow_tint,
            shift = util.by_pixel(0, 8*linescale),
            scale = linescale --1
          },
          east =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-E-glow.png",
            priority = "extra-high",
            width = 60,
            height = 62,
            shift = util.by_pixel(-22*linescale, -12*linescale),
            scale = linescale --1
          },
          south =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-S-glow.png",
            priority = "extra-high",
            width = 38,
            height = 40,
            tint = heat_glow_tint,
            shift = util.by_pixel(0, -36*linescale),
            scale = linescale --1
          },
          west =
          {
            filename = "__base__/graphics/entity/heat-exchanger/heatex-W-glow.png",
            priority = "extra-high",
            width = 60,
            height = 64,
            shift = util.by_pixel(20*linescale, -12*linescale),
            scale = linescale --1
          },
        }        
      },
      working_sound =
      {
        sound =
        {
          filename = "__base__/sound/boiler.ogg",
          volume = 0.8
        },
        max_sounds_per_type = 3
      },

      structure =
      {
        north =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/heat-exchanger/heatex-N-idle.png",
              priority = "extra-high",
              width = 131,
              height = 108,
              shift = util.by_pixel(-0.5*linescale, 4*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-N-idle.png",
                priority = "extra-high",
                width = 269,
                height = 221,
                shift = util.by_pixel(-1.25*linescale, 5.25*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-N-shadow.png",
              priority = "extra-high",
              width = 137,
              height = 82,
              scale = linescale, --1
              shift = util.by_pixel(20.5*linescale, 9*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-N-shadow.png",
                priority = "extra-high",
                width = 274,
                height = 164,
                scale = 0.5 * linescale,
                shift = util.by_pixel(20.5*linescale, 9*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        east =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/heat-exchanger/heatex-E-idle.png",
              priority = "extra-high",
              width = 102,
              height = 147,
              shift = util.by_pixel(-2*linescale, -0.5*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-E-idle.png",
                priority = "extra-high",
                width = 211,
                height = 301,
                shift = util.by_pixel(-1.75*linescale, 1.25*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-E-shadow.png",
              priority = "extra-high",
              width = 92,
              height = 97,
              scale = linescale, --1
              shift = util.by_pixel(30*linescale, 9.5*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-E-shadow.png",
                priority = "extra-high",
                width = 184,
                height = 194,
                scale = 0.5 * linescale,
                shift = util.by_pixel(30*linescale, 9.5*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        south =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/heat-exchanger/heatex-S-idle.png",
              priority = "extra-high",
              width = 128,
              height = 100,
              shift = util.by_pixel(3*linescale, 10*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-S-idle.png",
                priority = "extra-high",
                width = 260,
                height = 201,
                shift = util.by_pixel(4*linescale, 10.75*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-S-shadow.png",
              priority = "extra-high",
              width = 156,
              height = 66,
              scale = linescale, --1
              shift = util.by_pixel(30*linescale, 16*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-S-shadow.png",
                priority = "extra-high",
                width = 311,
                height = 131,
                scale = 0.5 * linescale,
                shift = util.by_pixel(29.75*linescale, 15.75*linescale),
                draw_as_shadow = true
              }
            }
          }
        },
        west =
        {
          layers =
          {
            {
              filename = "__base__/graphics/entity/heat-exchanger/heatex-W-idle.png",
              priority = "extra-high",
              width = 96,
              height = 132,
              shift = util.by_pixel(1*linescale, 5*linescale),
              scale = linescale, --1
              hr_version =
          {
                filename = "__base__/graphics/entity/heat-exchanger/hr-heatex-W-idle.png",
                priority = "extra-high",
                width = 196,
                height = 273,
                shift = util.by_pixel(1.5*linescale, 7.75*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/boiler/boiler-W-shadow.png",
              priority = "extra-high",
              width = 103,
              height = 109,
              scale = linescale, --1
              shift = util.by_pixel(19.5*linescale, 6.5*linescale),
              draw_as_shadow = true,
              hr_version =
          {
                filename = "__base__/graphics/entity/boiler/hr-boiler-W-shadow.png",
                priority = "extra-high",
                width = 206,
                height = 218,
                scale = 0.5 * linescale,
                shift = util.by_pixel(19.5*linescale, 6.5*linescale),
                draw_as_shadow = true
              }
            }
          }
        }
      },
      fire = {},
      fire_glow = {},
      burning_cooldown = 20
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        -- {"speed-module-3", module_ing},
        -- {"productivity-module-3", module_ing},
        -- {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-o-b"
    },
  }
end
