local tier_max = settings.startup["machinescaling-tier-lab"].value

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

enty_type = "lab"
machine_name = "lab"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "automation-3" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 100 * math.pow(2,tier)

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "production-machine",
      order = "g[lab]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-player", "player-creation"},
      minable = {mining_time = 0.2, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_scale = linescale-1,
      collision_box = {{0.3-1.5*linescale, 0.3-1.5*linescale}, {-0.3+1.5*linescale, -0.3+1.5*linescale}},
      -- collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      light = {intensity = 0.75, size = 8, color = {r = 1.0, g = 1.0, b = 1.0}},
      on_animation =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/lab/lab.png",
            width = 98,
            height = 87,
            frame_count = 33,
            line_length = 11,
            animation_speed = 1 / 3,
            shift = util.by_pixel(0, 1.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab.png",
              width = 194,
              height = 174,
              frame_count = 33,
              line_length = 11,
              animation_speed = 1 / 3,
              shift = util.by_pixel(0, 1.5*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/lab/lab-integration.png",
            width = 122,
            height = 81,
            frame_count = 1,
            line_length = 1,
            repeat_count = 33,
            animation_speed = 1 / 3,
            shift = util.by_pixel(0, 15.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab-integration.png",
              width = 242,
              height = 162,
              frame_count = 1,
              line_length = 1,
              repeat_count = 33,
              animation_speed = 1 / 3,
              shift = util.by_pixel(0, 15.5*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/lab/lab-shadow.png",
            width = 122,
            height = 68,
            frame_count = 1,
            line_length = 1,
            repeat_count = 33,
            animation_speed = 1 / 3,
            shift = util.by_pixel(13*linescale, 11*linescale),
            scale = linescale, --1
            draw_as_shadow = true,
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab-shadow.png",
              width = 242,
              height = 136,
              frame_count = 1,
              line_length = 1,
              repeat_count = 33,
              animation_speed = 1 / 3,
              shift = util.by_pixel(13*linescale, 11*linescale),
              scale = 0.5 * linescale,
              draw_as_shadow = true
            }
          }
        }
      },
      off_animation =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/lab/lab.png",
            width = 98,
            height = 87,
            frame_count = 1,
            shift = util.by_pixel(0, 1.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab.png",
              width = 194,
              height = 174,
              frame_count = 1,
              shift = util.by_pixel(0, 1.5*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/lab/lab-integration.png",
            width = 122,
            height = 81,
            frame_count = 1,
            shift = util.by_pixel(0, 15.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab-integration.png",
              width = 242,
              height = 162,
              frame_count = 1,
              shift = util.by_pixel(0, 15.5*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/lab/lab-shadow.png",
            width = 122,
            height = 68,
            frame_count = 1,
            shift = util.by_pixel(13*linescale, 11*linescale),
            scale = linescale, --1
            draw_as_shadow = true,
            hr_version =
            {
              filename = "__base__/graphics/entity/lab/hr-lab-shadow.png",
              width = 242,
              height = 136,
              frame_count = 1,
              shift = util.by_pixel(13*linescale, 11*linescale),
              draw_as_shadow = true,
              scale = 0.5 * linescale
            }
          }
        }
      },
      working_sound =
      {
        sound =
        {
          filename = "__base__/sound/lab.ogg",
          volume = 0.7
        },
        apparent_volume = 1
      },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input"
      },
      energy_usage = 60*multscale .. "kW", --"60kW",
      researching_speed = 1 * multscale,
      -- inputs =
      -- {
      --   "automation-science-pack",
      --   "logistic-science-pack",
      --   "chemical-science-pack",
      --   "military-science-pack",
      --   "production-science-pack",
      --   "utility-science-pack",
      --   "space-science-pack"
      -- },
      inputs = data.raw[enty_type][machine_name].inputs,
      module_specification =
      {
        module_slots = 2,
        max_entity_info_module_icons_per_row = 3,
        max_entity_info_module_icon_rows = 1,
        module_info_icon_shift = {0, 0.9*linescale},
        module_info_icon_scale = 0.5 * linescale
      }
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-d"
    },
  }
end
