local tier_max = settings.startup["machinescaling-tier-nuclear-reactor"].value

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local connections_H
local cons_xa, cons_xd
local con_y

enty_type = "reactor"
machine_name = "nuclear-reactor"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
-- module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "nuclear-power" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 300 * math.pow(2,tier)
  connections_H = {}
  con_y = 2.5*tier + 2
      -- ori: 2
      -- +1:  4.5
      -- +2:  7
  cons_xa = {}
  cons_xd = {}
      -- ori: 2, 0, -2
      -- +1:  4.5, 0.5, -3.5
      -- +2:  6, 0, 6
  for i = 0,2,1 do
    cons_xd[i] = 2.5*tier + 2 - (2+2*tier)*i - math.floor(tier*0.5)
    cons_xa[i] = -cons_xd[i]
  end
  for _,con_x in pairs(cons_xa) do
    table.insert( connections_H,  { position = { con_x , -con_y }, direction = defines.direction.north } )
  end
  for _,con_x in pairs(cons_xa) do
    table.insert( connections_H,  { position = { con_y ,  con_x }, direction = defines.direction.east } )
  end
  for _,con_x in pairs(cons_xa) do
    table.insert( connections_H,  { position = { con_x ,  con_y }, direction = defines.direction.south } )
  end
  for _,con_x in pairs(cons_xa) do
    table.insert( connections_H,  { position = {-con_y ,  con_x }, direction = defines.direction.west } )
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "energy",
      order = "f[nuclear-energy]-a[reactor]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },

    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "player-creation"},
      minable = {mining_time = 0.5, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      alert_icon_scale = linescale-1,
      -- scale_entity_info_icon = true,
      consumption = 40*multscale .. "MW", --"40MW",
      neighbour_bonus = 1,
      energy_source =
      {
        type = "burner",
        fuel_category = "nuclear",
        effectivity = 1,
        fuel_inventory_size = 1,
        burnt_inventory_size = 1
      },
      collision_box = {{0.3-2.5*linescale, 0.3-2.5*linescale}, {-0.3+2.5*linescale, -0.3+2.5*linescale}},
      -- collision_box = {{-2.2, -2.2}, {2.2, 2.2}},
      selection_box = {{-2.5*linescale, -2.5*linescale}, {2.5*linescale, 2.5*linescale}},
      -- selection_box = {{-2.5, -2.5}, {2.5, 2.5}},
      --the collision box of a reactor is increased by this on the sides where it connects to another reactor:
      --neighbour_collision_increase = 0.25,
      lower_layer_picture =
      {
        filename = "__base__/graphics/entity/nuclear-reactor/reactor-pipes.png",
        width = 156,
        height = 156,
        scale = linescale, --1
        shift = util.by_pixel(-2*linescale, -4*linescale),
        hr_version =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-pipes.png",
          width = 320,
          height = 316,
          scale = 0.5 * linescale,
          shift = util.by_pixel(-1*linescale, -5*linescale),
        }
      },
      heat_lower_layer_picture =
      {
        filename = "__base__/graphics/entity/nuclear-reactor/reactor-pipes-heated.png",
        width = 156,
        height = 156,
        scale = linescale, --1
        shift = util.by_pixel(-3*linescale, -4*linescale),
        hr_version =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-pipes-heated.png",
          width = 320,
          height = 316,
          scale = 0.5 * linescale,
          shift = util.by_pixel(-0.5*linescale, -4.5*linescale),
        }
      },

      picture =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/nuclear-reactor/reactor.png",
            width = 154,
            height = 158,
            scale = linescale, --1
            shift = util.by_pixel(-6*linescale, -6*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor.png",
              width = 302,
              height = 318,
              scale = 0.5 * linescale,
              shift = util.by_pixel(-5*linescale, -7*linescale),
            }
          },
          {
            filename = "__base__/graphics/entity/nuclear-reactor/reactor-shadow.png",
            width = 263,
            height = 162,
            scale = linescale, --1
            shift = { 1.625*linescale , 0 },
            draw_as_shadow = true,
            hr_version =
            {
              filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-shadow.png",
              width = 525,
              height = 323,
              scale = 0.5 * linescale,
              shift = { 1.625*linescale, 0 },
              draw_as_shadow = true
            }
          }
        }
      },

      working_light_picture =
      {
        filename = "__base__/graphics/entity/nuclear-reactor/reactor-lights-color.png",
        width = 160,
        height = 160,
        scale = linescale, --1
        shift = { -0.03125*linescale, -0.1875*linescale },
        blend_mode = "additive",
        hr_version =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-lights-color.png",
          width = 320,
          height = 320,
          scale = 0.5 * linescale,
          shift = { -0.03125*linescale, -0.1875*linescale },
          blend_mode = "additive"
        }
      },

      light = {intensity = 0.6, size = 9.9, shift = {0.0, 0.0}, color = {r = 0.0, g = 1.0, b = 0.0}},

      heat_buffer =
      {
        max_temperature = 1000,
        specific_heat = 10*multscale .. "MJ", --"10MJ",
        max_transfer = 10*multscale .. "GW", --"10GW",
        minimum_glow_temperature = 350,
        glow_alpha_modifier = 0.6,
        connections = connections_H,
        -- connections =
        -- {
        --   {
        --     position = {-2, -2},
        --     direction = defines.direction.north
        --   },
        --   {
        --     position = {0, -2},
        --     direction = defines.direction.north
        --   },
        --   {
        --     position = {2, -2},
        --     direction = defines.direction.north
        --   },
        --   {
        --     position = {2, -2},
        --     direction = defines.direction.east
        --   },
        --   {
        --     position = {2, 0},
        --     direction = defines.direction.east
        --   },
        --   {
        --     position = {2, 2},
        --     direction = defines.direction.east
        --   },
        --   {
        --     position = {2, 2},
        --     direction = defines.direction.south
        --   },
        --   {
        --     position = {0, 2},
        --     direction = defines.direction.south
        --   },
        --   {
        --     position = {-2, 2},
        --     direction = defines.direction.south
        --   },
        --   {
        --     position = {-2, 2},
        --     direction = defines.direction.west
        --   },
        --   {
        --     position = {-2, 0},
        --     direction = defines.direction.west
        --   },
        --   {
        --     position = {-2, -2},
        --     direction = defines.direction.west
        --   }
        -- }
        heat_picture =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-heated.png",
          width = 108,
          height = 128,
          scale = linescale, --1
          shift = util.by_pixel(1*linescale, -7*linescale),
          hr_version =
          {
            filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-heated.png",
            width = 216,
            height = 256,
            scale = 0.5 * linescale,
            shift = util.by_pixel(3*linescale, -6.5*linescale),
          }
        },
        heat_glow =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-heat-glow.png",
          priority = "extra-high",
          width = 188,
          height = 190,
          tint = heat_glow_tint,
          scale = linescale, --1
          shift = util.by_pixel(-2*linescale, -4*linescale)
        }
      },

      connection_patches_connected =
      {
        sheet =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-connect-patches.png",
          width = 32,
          height = 32,
          variation_count = 12,
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-connect-patches.png",
            width = 64,
            height = 64,
            variation_count = 12,
            scale = 0.5 * linescale
          }
        }
      },

      connection_patches_disconnected =
      {
        sheet =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-connect-patches.png",
          width = 32,
          height = 32,
          variation_count = 12,
          y = 32,
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-connect-patches.png",
            width = 64,
            height = 64,
            variation_count = 12,
            y = 64,
            scale = 0.5 * linescale
          }
        }
      },

      -- connection_patches =
      -- {
      --   north =
      --   {
      --     filename = "__base__/graphics/entity/nuclear-reactor/connection-patch-north.png",
      --     width = 160,
      --     height = 15,
      --     shift = util.by_pixel(0, -72.5*linescale)
      --   },
      --   east =
      --   {
      --     filename = "__base__/graphics/entity/nuclear-reactor/connection-patch-east.png",
      --     width = 15,
      --     height = 160,
      --     shift = util.by_pixel(72.5*linescale, 0)
      --   },
      --   south =
      --   {
      --     filename = "__base__/graphics/entity/nuclear-reactor/connection-patch-south.png",
      --     width = 160,
      --     height = 15,
      --     shift = util.by_pixel(0, 72.5*linescale)
      --   },
      --   west =
      --   {
      --     filename = "__base__/graphics/entity/nuclear-reactor/connection-patch-west.png",
      --     width = 15,
      --     height = 160,
      --     shift = util.by_pixel(-72.5*linescale, 0)
      --   }
      -- },

      heat_connection_patches_connected =
      {
        sheet =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-connect-patches-heated.png",
          width = 32,
          height = 32,
          variation_count = 12,
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-connect-patches-heated.png",
            width = 64,
            height = 64,
            variation_count = 12,
            scale = 0.5 * linescale
          }
        }
      },

      heat_connection_patches_disconnected =
      {
        sheet =
        {
          filename = "__base__/graphics/entity/nuclear-reactor/reactor-connect-patches-heated.png",
          width = 32,
          height = 32,
          variation_count = 12,
          y = 32,
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/nuclear-reactor/hr-reactor-connect-patches-heated.png",
            width = 64,
            height = 64,
            variation_count = 12,
            y = 64,
            scale = 0.5 * linescale
          }
        }
      },

      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65},

      meltdown_action =
      {
        type = "direct",
        action_delivery =
        {
          type = "instant",
          target_effects =
          {
            {
                repeat_count = 100,
                type = "create-trivial-smoke",
                smoke_name = "nuclear-smoke",
                offset_deviation = {{-1, -1}, {1, 1}},
                starting_frame = 3,
                starting_frame_deviation = 5,
                starting_frame_speed = 0,
                starting_frame_speed_deviation = 5,
                speed_from_center = 0.5
            },
            {
              type = "create-entity",
              entity_name = "explosion"
            },
            {
              type = "damage",
              damage = {amount = 400*multscale, type = "explosion"}
            },
            {
              type = "create-entity",
              entity_name = "small-scorchmark",
              check_buildability = true
            },
            {
              type = "nested-result",
              action =
              {
                type = "area",
                target_entities = false,
                trigger_from_target = true,
                repeat_count = 2000+500*tier,
                radius = 35+5*tier,
                action_delivery =
                {
                  type = "projectile",
                  projectile = "atomic-bomb-wave",
                  starting_speed = 0.5
                }
              }
            }
          }
        }
      }
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        -- {"speed-module-3", module_ing},
        -- {"productivity-module-3", module_ing},
        -- {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-o-c"
    },
  }
end
