local tier_max = settings.startup["machinescaling-tier-oil-refinery"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_I1
local pipe_connections_I2
local pipe_connections_O1
local pipe_connections_O2
local pipe_connections_O3
local fluid_box_I
local fluid_box_O

enty_type = "assembling-machine"
machine_name = "oil-refinery"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "oil-processing" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 150 * math.pow(2,tier)
  pipe_connections_I1 = {}
  pipe_connections_I2 = {}
  pipe_connections_O1 = {}
  pipe_connections_O2 = {}
  pipe_connections_O3 = {}
  for i = 0,tier,1 do
    local pipe_xi = 1+0.5*tier + i
      -- ori: 1
      -- +1:  1.5, 2.5
      -- +2:  2, 3, 4
      -- +3:  2.5, 3.5, 4.5, 5.5
    local pipe_xo = 2+1.5*tier + i
      -- ori: 2
      -- +1:  3.5, 4.5
      -- +2:  4, 5, 6
      -- +3:  5.5, 6.5, 7.5, 8.5
    local pipe_xc = -0.5*tier + i
      -- ori: 0
      -- +1:  -0.5, 0.5
      -- +2:  -1, 0, 1
      -- +3:  1.5, -0.5, 0.5, 1.5
    table.insert( pipe_connections_I1, { type="input", position = {-pipe_xi,  0.5+2.5*linescale} } )
    table.insert( pipe_connections_I2, { type="input", position = { pipe_xi,  0.5+2.5*linescale} } )
    table.insert( pipe_connections_O1, { position = {-pipe_xo, -0.5-2.5*linescale} } )
    table.insert( pipe_connections_O2, { position = { pipe_xc, -0.5-2.5*linescale} } )
    table.insert( pipe_connections_O3, { position = { pipe_xo, -0.5-2.5*linescale} } )
  end
  if pipescaling then
    fluid_box_I =
    {
      base_area = 10,
      height = 1*multscale,
      base_level = -1*multscale,
    }
    fluid_box_O =
    {
      base_area = 1,
      height = 1*multscale,
      base_level = 1*multscale,
    }
  else
    fluid_box_I =
    {
      base_area = 10*multscale,
      height = 1,
      base_level = -1,
    }
    fluid_box_O =
    {
      base_area = 1*multscale,
      height = 1,
      base_level = 1,
    }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "production-machine",
      order = "d[refinery]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral","player-creation"},
      minable = {mining_time = 0.2, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_scale = linescale-1,
      scale_entity_info_icon = true,
      collision_box = {{0.1-2.5*linescale, 0.1-2.5*linescale}, {-0.1+2.5*linescale, -0.1+2.5*linescale}},
      -- collision_box = {{-2.4, -2.4}, {2.4, 2.4}},
      selection_box = {{-2.5*linescale, -2.5*linescale}, {2.5*linescale, 2.5*linescale}},
      -- selection_box = {{-2.5, -2.5}, {2.5, 2.5}},
      drawing_box = {{-2.5*linescale, -0.3-2.5*linescale}, {2.5*linescale, 2.5*linescale}},
      -- drawing_box = {{-2.5, -2.8}, {2.5, 2.5}},
      module_specification =
      {
        module_slots = 3,
        module_info_icon_shift = {0, 1.0*linescale},
        module_info_icon_scale = 0.5 * linescale
      },
      allowed_effects = {"consumption", "speed", "productivity", "pollution"},
      crafting_categories = {"oil-processing"},
      crafting_speed = 1 * multscale,
      has_backer_name = true,
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-input",
        emissions_per_minute = 6 * multscale
        -- emissions_per_second_per_watt = 6 / 420000
        -- emissions = 0.03 / 3.5
      },
      energy_usage = 420*multscale .. "kW", --"420kW",


      animation = make_4way_animation_from_spritesheet({ layers =
      {
        {
          filename = "__base__/graphics/entity/oil-refinery/oil-refinery.png",
          width = 337,
          height = 255,
          frame_count = 1,
          shift = {2.515625*linescale, 0.484375*linescale},
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/oil-refinery/hr-oil-refinery.png",
            width = 386,
            height = 430,
            frame_count = 1,
            shift = util.by_pixel(0, -7.5*linescale),
            scale = 0.5 * linescale
          }
        },
        {
          filename = "__base__/graphics/entity/oil-refinery/oil-refinery-shadow.png",
          width = 337,
          height = 213,
          frame_count = 1,
          shift = util.by_pixel(82.5*linescale, 26.5*linescale),
          draw_as_shadow = true,
          scale = linescale, --1
          hr_version =
          {
            filename = "__base__/graphics/entity/oil-refinery/hr-oil-refinery-shadow.png",
            width = 674,
            height = 426,
            frame_count = 1,
            shift = util.by_pixel(82.5*linescale, 26.5*linescale),
            draw_as_shadow = true,
            scale = 0.5 * linescale
          }
        }
      }}),
      working_visualisations =
      {
        {
          north_position = util.by_pixel(34*linescale, -65*linescale),
          east_position = util.by_pixel(-52*linescale, -61*linescale),
          south_position = util.by_pixel(-59*linescale, -82*linescale),
          west_position = util.by_pixel(57*linescale, -58*linescale),
          animation =
          {
            filename = "__base__/graphics/entity/oil-refinery/oil-refinery-fire.png",
            line_length = 10,
            width = 20,
            height = 40,
            frame_count = 60,
            animation_speed = 0.75,
            scale = linescale, --1
            animation_speed = 1/multscale,
            shift = util.by_pixel(0, -14*linescale),
            hr_version =
            {
              filename = "__base__/graphics/entity/oil-refinery/hr-oil-refinery-fire.png",
              line_length = 10,
              width = 40,
              height = 81,
              frame_count = 60,
              animation_speed = 0.75,
              scale = 0.5 * linescale,
              animation_speed = 1/multscale,
              shift = util.by_pixel(0, -14.25*linescale)
            }
          },
          light = {intensity = 0.4, size = 6, color = {r = 1.0, g = 1.0, b = 1.0}}
        }
      },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound = { filename = "__base__/sound/oil-refinery.ogg" },
        idle_sound = { filename = "__base__/sound/idle1.ogg", volume = 0.6 },
        apparent_volume = 2.5
      },
      fluid_boxes =
      {
        {
          production_type = "input",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_I.base_area,
          height = fluid_box_I.height,
          base_level = fluid_box_I.base_level,
          pipe_connections = pipe_connections_I1,
          -- pipe_connections = {{ type="input", position = {-1, 3} }}
        },
        {
          production_type = "input",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_I.base_area,
          height = fluid_box_I.height,
          base_level = fluid_box_I.base_level,
          pipe_connections = pipe_connections_I2,
          -- pipe_connections = {{ type="input", position = {1, 3} }}
        },
        {
          production_type = "output",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O1,
          -- pipe_connections = {{ position = {-2, -3} }}
        },
        {
          production_type = "output",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O2,
          -- pipe_connections = {{ position = {0, -3} }}
        },
        {
          production_type = "output",
          pipe_covers = pipecoverspictures(),
          base_area = fluid_box_O.base_area,
          height = fluid_box_O.height,
          base_level = fluid_box_O.base_level,
          pipe_connections = pipe_connections_O3,
          -- pipe_connections = {{ position = {2, -3} }}
        }
      },
      pipe_covers = pipecoverspictures()
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        {"speed-module-3", module_ing},
        {"productivity-module-3", module_ing},
        {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-a-c"
    },
  }
end
