local tier_max = settings.startup["machinescaling-tier-steam-turbine"].value
local pipescaling = mods["SchallPipeScaling"]

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local base_icons_path
local icon_t
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_IO
local fluid_box

enty_type = "generator"
machine_name = "steam-turbine"
base_icon_path = data.raw.item[machine_name].icon
base_icons_path = data.raw.item[machine_name].icons

tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
-- module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "nuclear-power" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 200 * math.pow(2,tier)
  pipe_connections_IO = {}
  for i = 0,tier,1 do
    local pipe_xio = -0.5*tier + i
      -- ori: 0
      -- +1:  -0.5, 0.5
      -- +2:  -1, 0, 1
      -- +3:  -1.5, -0.5, 0.5, 1.5
    table.insert( pipe_connections_IO, { type = "input-output", position = {pipe_xio,  0.5+2.5*linescale} } )
    table.insert( pipe_connections_IO, { type = "input-output", position = {pipe_xio, -0.5-2.5*linescale} } )
  end
  if pipescaling then
    fluid_box =
    {
      base_area = 1,
      height = 2*multscale,
      base_level = -1*multscale,
    }
  else
    fluid_box =
    {
      base_area = 1*multscale,
      height = 2,
      base_level = -1,
    }
  end

  icon_t = {}
  if base_icon_path then
    icon_t[1] = { icon = base_icon_path }
    icon_t[2] = { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint }
  else
    icon_t[1] = { icon = base_icons_path[1].icon }
    icon_t[2] = { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint }
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = icon_t,
      icon_size = 32,
      subgroup = "energy",
      order = "b[steam-power]-c[steam-turbine]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = icon_t,
      icon_size = 32,
      flags = {"placeable-neutral", "player-creation"},
      minable = {mining_time = 0.3, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "big-remnants",
      dying_explosion = "medium-explosion",
      alert_icon_shift = util.by_pixel(0, -12*linescale),
      alert_icon_scale = linescale-1,
      -- scale_entity_info_icon = true,
      effectivity = 1,
      fluid_usage_per_tick = 1*multscale,
      maximum_temperature = 500,
      burns_fluid = false,
      resistances =
      {
        {
          type = "fire",
          percent = 70
        }
      },
      fast_replaceable_group = "steam-engine",
      collision_box = {{0.15-1.5*linescale, 0.15-2.5*linescale}, {-0.15+1.5*linescale, -0.15+2.5*linescale}},
      -- collision_box = {{-1.35, -2.35}, {1.35, 2.35}},
      selection_box = {{-1.5*linescale, -2.5*linescale}, {1.5*linescale, 2.5*linescale}},
      -- selection_box = {{-1.5, -2.5}, {1.5, 2.5}},
      fluid_box =
      {
        base_area = fluid_box.base_area,
        height = fluid_box.height,
        base_level = fluid_box.base_level,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_IO,
        -- pipe_connections =
        -- {
        --   { type = "input-output", position = {0, 3} },
        --   { type = "input-output", position = {0, -3} }
        -- },
        production_type = "input-output",
        filter = "steam",
        minimum_temperature = 100.0
      },
      energy_source =
      {
        type = "electric",
        usage_priority = "secondary-output"
      },
      horizontal_animation =
      {
        layers =
        {
          {
            filename = "__base__/graphics/entity/steam-turbine/steam-turbine-H.png",
            width = 160,
            height = 123,
            frame_count = 8,
            line_length = 4,
            shift = util.by_pixel(0, -2.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/steam-turbine/hr-steam-turbine-H.png",
              width = 320,
              height = 245,
              frame_count = 8,
              line_length = 4,
              shift = util.by_pixel(0, -2.75*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/steam-turbine/steam-turbine-H-shadow.png",
            width = 217,
            height = 74,
            repeat_count = 8,
            frame_count = 1,
            line_length = 1,
            draw_as_shadow = true,
            shift = util.by_pixel(28.75*linescale, 18*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/steam-turbine/hr-steam-turbine-H-shadow.png",
              width = 435,
              height = 150,
              repeat_count = 8,
              frame_count = 1,
              line_length = 1,
              draw_as_shadow = true,
              shift = util.by_pixel(28.5*linescale, 18*linescale),
              scale = 0.5 * linescale
            }
          }
        }
      },
      vertical_animation =
      {
       layers =
       {
          {
            filename = "__base__/graphics/entity/steam-turbine/steam-turbine-V.png",
            width = 108,
            height = 173,
            frame_count = 8,
            line_length = 4,
            shift = util.by_pixel(5*linescale, 6.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/steam-turbine/hr-steam-turbine-V.png",
              width = 217,
              height = 347,
              frame_count = 8,
              line_length = 4,
              shift = util.by_pixel(4.75*linescale, 6.75*linescale),
              scale = 0.5 * linescale
            }
          },
          {
            filename = "__base__/graphics/entity/steam-turbine/steam-turbine-V-shadow.png",
            width = 151,
            height = 131,
            repeat_count = 8,
            frame_count = 1,
            line_length = 1,
            draw_as_shadow = true,
            shift = util.by_pixel(39.5*linescale, 24.5*linescale),
            scale = linescale, --1
            hr_version =
            {
              filename = "__base__/graphics/entity/steam-turbine/hr-steam-turbine-V-shadow.png",
              width = 302,
              height = 260,
              repeat_count = 8,
              frame_count = 1,
              line_length = 1,
              draw_as_shadow = true,
              shift = util.by_pixel(39.5*linescale, 24.5*linescale),
              scale = 0.5 * linescale
            }
          }
        }
      },
      smoke =
      {
        {
          name = "turbine-smoke",
          north_position = {0.0, -1.0*linescale},
          east_position = {0.75*linescale, -0.75*linescale},
          frequency = 10 / 32,
          starting_vertical_speed = 0.08,
          slow_down_factor = 1,
          starting_frame_deviation = 60
        }
      },
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
          filename = "__base__/sound/steam-engine-90bpm.ogg",
          volume = 0.6
        },
        match_speed_to_activity = true
      },
      min_perceived_performance = 0.25,
      performance_to_sound_speedup = 0.5
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        -- {"speed-module-3", module_ing},
        -- {"productivity-module-3", module_ing},
        -- {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-p-b"
    },
  }
end
