local tier_max = settings.startup["machinescaling-tier-storage-tank"].value

local linescale
local multscale
local enty_type
local machine_name
local machine_name_LS
local machine_name_MS
local base_icon_path
local overtier_icon_prefix = "__SchallMachineScaling__/graphics/icons/overtier-"
local overtier_icon_suffix = ".png"
local overtier_icon_tint = {r=1.0, g=1.0, b=0.0, a=1}
local module_ing
local tech_icon_path
local tech_MS_icon_path = "__SchallMachineScaling__/graphics/technology/overtier.png"
local prerequisites
local science_count

local pipe_connections_IO

enty_type = "storage-tank"
machine_name = "storage-tank"
base_icon_path = data.raw.item[machine_name].icon
tech_icon_path = "__SchallMachineScaling__/graphics/technology/"..machine_name..".png"
-- module_ing = data.raw[enty_type][machine_name].module_specification.module_slots
for tier = 1,tier_max,1 do
  linescale = tier+1
  multscale = math.pow(4,tier)
  if tier == 1 then
    machine_name_LS = machine_name
    prerequisites = { "fluid-handling" }
  else
    machine_name_LS = machine_name.."-MS-"..tier-1
    prerequisites = { machine_name_LS }
  end
  machine_name_MS = machine_name.."-MS-"..tier
  science_count = 50 * math.pow(2,tier)
  pipe_connections_IO = {}
  for i = 0,tier,1 do
    local pipe_x = 1.5*tier + 1 - i
      -- ori: 1
      -- +1:  2.5, 1.5
      -- +2:  4, 3, 2
      -- +3:  5.5, 4.5, 3.5, 2.5
    table.insert( pipe_connections_IO, { position = {-pipe_x, -0.5-1.5*linescale} } )
    table.insert( pipe_connections_IO, { position = {  0.5+1.5*linescale, pipe_x} } )
    table.insert( pipe_connections_IO, { position = { pipe_x,  0.5+1.5*linescale} } )
    table.insert( pipe_connections_IO, { position = { -0.5-1.5*linescale,-pipe_x} } )
  end

  data:extend
  {
    {
      type = "item",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      subgroup = "storage",
      order = "b[fluid]-a[storage-tank]-"..tier,
      place_result = machine_name_MS,
      stack_size = 10
    },
    {
      type = enty_type,
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icons = { { icon = base_icon_path },
                { icon = overtier_icon_prefix..tier..overtier_icon_suffix, tint = overtier_icon_tint } },
      icon_size = 32,
      flags = {"placeable-neutral", "player-creation"},
      minable = {mining_time = 0.5, result = machine_name_MS},
      max_health = data.raw[enty_type][machine_name].max_health * linescale,
      corpse = "medium-remnants",
      alert_icon_shift = util.by_pixel(3*linescale, -34*linescale),
      alert_icon_scale = linescale-1,
      -- scale_entity_info_icon = true,
      collision_box = {{0.2-1.5*linescale, 0.2-1.5*linescale}, {-0.2+1.5*linescale, -0.2+1.5*linescale}},
      -- collision_box = {{-1.3, -1.3}, {1.3, 1.3}},
      selection_box = {{-1.5*linescale, -1.5*linescale}, {1.5*linescale, 1.5*linescale}},
      -- selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
      fluid_box =
      {
        base_area = 250*multscale,
        pipe_covers = pipecoverspictures(),
        pipe_connections = pipe_connections_IO
        -- pipe_connections =
        -- {
        --   { position = {-1, -2} },
        --   { position = {2, 1} },
        --   { position = {1, 2} },
        --   { position = {-2, -1} }
        -- }
      },
      two_direction_only = true,
      window_bounding_box = {{-0.125*linescale, 0.6875*linescale}, {0.1875*linescale, 1.1875*linescale}},
      pictures =
      {
        picture =
        {
          sheets =
          {
            {
              filename = "__base__/graphics/entity/storage-tank/storage-tank.png",
              priority = "extra-high",
              frames = 2,
              width = 110,
              height = 108,
              shift = util.by_pixel(0, 4*linescale),
              scale = linescale, --1
              hr_version =
              {
                filename = "__base__/graphics/entity/storage-tank/hr-storage-tank.png",
                priority = "extra-high",
                frames = 2,
                width = 219,
                height = 215,
                shift = util.by_pixel(-0.25*linescale, 3.75*linescale),
                scale = 0.5 * linescale
              }
            },
            {
              filename = "__base__/graphics/entity/storage-tank/storage-tank-shadow.png",
              priority = "extra-high",
              frames = 2,
              width = 146,
              height = 77,
              shift = util.by_pixel(30*linescale, 22.5*linescale),
              scale = linescale, --1
              draw_as_shadow = true,
              hr_version =
              {
                filename = "__base__/graphics/entity/storage-tank/hr-storage-tank-shadow.png",
                priority = "extra-high",
                frames = 2,
                width = 291,
                height = 153,
                shift = util.by_pixel(29.75*linescale, 22.25*linescale),
                scale = 0.5 * linescale,
                draw_as_shadow = true
              }
            }
          }
        },
        fluid_background =
        {
          filename = "__base__/graphics/entity/storage-tank/fluid-background.png",
          priority = "extra-high",
          width = 32,
          height = 15,
        },
        window_background =
        {
          filename = "__base__/graphics/entity/storage-tank/window-background.png",
          priority = "extra-high",
          width = 17,
          height = 24,
          hr_version =
          {
            filename = "__base__/graphics/entity/storage-tank/hr-window-background.png",
            priority = "extra-high",
            width = 34,
            height = 48,
            scale = 0.5
          }
        },
        flow_sprite =
        {
          filename = "__base__/graphics/entity/pipe/fluid-flow-low-temperature.png",
          priority = "extra-high",
          width = 160,
          height = 20
        },
        gas_flow =
        {
          filename = "__base__/graphics/entity/pipe/steam.png",
          priority = "extra-high",
          line_length = 10,
          width = 24,
          height = 15,
          frame_count = 60,
          axially_symmetrical = false,
          direction_count = 1,
          animation_speed = 0.25,
          hr_version =
          {
            filename = "__base__/graphics/entity/pipe/hr-steam.png",
            priority = "extra-high",
            line_length = 10,
            width = 48,
            height = 30,
            frame_count = 60,
            axially_symmetrical = false,
            animation_speed = 0.25,
            direction_count = 1
          }
        }
      },
      flow_length_in_ticks = 360,
      vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
      working_sound =
      {
        sound =
        {
            filename = "__base__/sound/storage-tank.ogg",
            volume = 0.8
        },
        match_volume_to_activity = true,
        apparent_volume = 1.5,
        max_sounds_per_type = 3
      },

      circuit_wire_connection_points = circuit_connector_definitions["storage-tank"].points,
      circuit_connector_sprites = circuit_connector_definitions["storage-tank"].sprites,
      circuit_wire_max_distance = default_circuit_wire_max_distance + 2*tier
    },
    {
      type = "recipe",
      name = machine_name_MS,
      enabled = false,
      energy_required = 10 * tier,
      ingredients =
      {
        {machine_name_LS, 4},
        -- {"speed-module-3", module_ing},
        -- {"productivity-module-3", module_ing},
        -- {"effectivity-module-3", module_ing}
      },
      result = machine_name_MS
    },
    {
      type = "technology",
      name = machine_name_MS,
      localised_name = { "entity-name.machinescaling-localised-name", {"entity-name."..machine_name}, tier },
      localised_description = { "entity-description.machinescaling-localised-description", {"entity-name."..machine_name}, multscale },
      icon_size = 128,
      icons = { { icon = tech_icon_path },
                { icon = tech_MS_icon_path } },
      effects =
      {
        {
          type = "unlock-recipe",
          recipe = machine_name_MS
        }
      },
      prerequisites = prerequisites,
      unit =
      {
        count = science_count,
        ingredients =
        {
          {"automation-science-pack", 1},
          {"logistic-science-pack", 1},
          {"chemical-science-pack", 1},
          {"production-science-pack", 1}
        },
        time = 30
      },
      upgrade = true,
      order = "ms-t-a"
    },
  }
end
